(* 
 * Copyright (c) 2001, 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: gontlink.mli,v 1.3 2002/08/12 12:42:12 malekith Exp $
 *
 * Module interface: Gontlink.
 * Description: Gont linker.
 *)

open Gontmodule

type link_info = 
  {
    (* Same as ci_target_prefix, needed since we want to generate Ksi
     * code. *)
    li_target_prefix: string;
    (* List of .go/.ga files to link. Bool argument, if true, means greedy,
     * that is all files from .ga are linked, regardless of whatever they 
     * are or are not used. Otherwise, modules from .ga files are linked
     * only if required by others. *)
    li_file_list: (bool * string) list;
    (* List of directories in which to search for .ga files. *)
    li_search_path: string list;
    (* Name of auxiliary Ksi file to generate. *)
    li_aux_ksi_file: string;
    (* Names of functions in auxiliary ksi file. *)
    li_init_name: string;
    li_fini_name: string;
    li_main_name: string;    (* Can be empty (no main is generated then). *)
    (* Options to be passed to C linker. *)
    li_ldopts: string;
  }

(* Make .ga archive from set of .go files. *)
val make_archive : (* target *) string -> (* ldopts *) string -> 
                   (* files *) string list -> unit

(* Generate auxiliary linking file, find module dependencies, return list
 * of files to pass to external linker (excluding auxiliary Ksi file,
 * that needs to be compiled anyway). *)
val link : link_info -> ((* ldopts *) string * string list)
