/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: c_string.c,v 1.2 2002/08/12 12:42:21 malekith Exp $
 *
 * Standard library for Gont, String module C implementation.
 */

/* This is for memrchr. */
#define _GNU_SOURCE

#include <gont/runtime.h>
#include <string.h>
#include <ctype.h>

const char *String__to_cstring(struct gont_string *s)
{
	return s->data;
}

gont_ptroff String__length(struct gont_string *s)
{
	return s->len;
}

struct gont_string *String__copy(struct gont_string *s)
{
	return gont_make_string(s->len, s->data);
}

struct gont_string *String__of_cstring(const char *s)
{
	if (s == NULL)
		gont_invalid_argument("String::of_cstring");

	/* no gont_cstring() for efficiency */
	return gont_make_string(strlen(s), s);
}

struct gont_string *String__sub(struct gont_string *s, gont_ptroff pos, 
				gont_ptroff len)
{
	/* Note that first check isn't redundant (overflow). */
	if (pos > s->len || pos + len > s->len)
		gont_invalid_argument("String::sub");
		
	return gont_make_string(len, s->data + pos);
}


gont_ptroff String__get(struct gont_string *s, gont_ptroff idx)
{
	if (idx >= s->len)
		gont_invalid_argument("String::get");
		
	return (unsigned char)s->data[idx];
}

void String__set(struct gont_string *s, gont_ptroff idx, gont_ptroff c)
{
	if (idx >= s->len)
		gont_invalid_argument("String::set");
		
	s->data[idx] = c;
}

struct gont_string *String__create(gont_ptroff n)
{
	return gont_make_string(n, NULL);
}

void String__fill(struct gont_string *s, gont_ptroff pos, 
		  gont_ptroff len, gont_ptroff c)
{
	if (pos > s->len || pos + len > s->len)
		gont_invalid_argument("String::fill");
		
	memset(s->data + pos, c, len);	
}

void String__blit(struct gont_string *src, gont_ptroff srcoff, 
		  struct gont_string *dst, gont_ptroff dstoff, 
		  gont_ptroff len)
{
	if (srcoff > src->len || dstoff > dst->len ||
	    srcoff + len > src->len || dstoff + len > dst->len)
		gont_invalid_argument("String::blit");
	memmove(dst->data + dstoff, src->data + srcoff, len);
}

gont_ptroff String___index_from(struct gont_string *s, gont_ptroff pos, 
				gont_ptroff c)
{
	char *p;
	
	if (pos >= s->len)
		return -2;
		
	p = memchr(s->data + pos, c, s->len - pos);

	if (p)
		return p - s->data;
	else
		return -1;
}

gont_ptroff String___rindex_from(struct gont_string *s, gont_ptroff pos, 
				 gont_ptroff c)
{
	char *p;
	
	if (pos >= s->len)
		return -2;
		
	p = memrchr(s->data, c, pos + 1);

	if (p)
		return p - s->data;
	else
		return -1;
}

gont_ptroff String___index_str_from(struct gont_string *s, gont_ptroff pos, 
				    struct gont_string *c)
{
	char *p;
	int len;
	
	if (pos > s->len)
		return -2;
		
	if (c->len == 0)
		return pos;
	
	len = s->len - pos - c->len + 1;

	if (len < 0)
		return -1;
	
	for (p = s->data + pos; len--; p++)
		if (memcmp(p, c->data, c->len) == 0)
			return p - s->data;

	return -1;
}


gont_ptroff String__uppercase_char(gont_ptroff c)
{
	return toupper(c & 0xff);
}

gont_ptroff String__lowercase_char(gont_ptroff c)
{
	return tolower(c & 0xff);
}
