/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: list.g,v 1.5 2002/08/12 12:42:21 malekith Exp $
 *
 * Standard library for Gont, List module implementation.
 */

open List

int length(<'a>t l)
{
	int length_rec(int res, <'a>t li)
	{
		switch li {
		case Nil[]:
			return res;
		case Cons[hd, tl]:
			return length_rec(res + 1, tl);
		}
	}

	return length_rec(0, l);
}

void iter(*(void ('a)) f, <'a>t l)
{
	switch l {
	case Nil[]:
		return;
		
	case Cons[hd, tl]:
		f(hd);
		iter(f, tl);
	}
}

<'b>t map(*('b ('a)) f, <'a>t l)
{
	switch l {
	case Nil[]:
		return Nil[];		

	case Cons[hd, tl]:
		return Cons[f(hd), map(f, tl)];
	}
}

<'a>t concat(<'a>t x, <'a>t y)
{
	switch x {
	case Nil[]:
		return y;
	case Cons[hd, tl]:
		return Cons[hd, concat(tl, y)];
	}
}

<'a>t filter(*(bool ('a)) f, <'a>t l)
{
	switch l {
	case Nil[]:
		return Nil[];
	case Cons[x, xs]:
		if (f(x)) {
			return Cons[x, filter(f, xs)];
		} else {
			return filter(f, xs);
		}
	}
	
}

'a fold_left (*('a ('a, 'b)) f, 'a init, <'b>t li) 
{
	switch li {
	case Nil[]: 
		return init;
	case Cons[hd, tl]: 
		return fold_left(f, f(init, hd), tl);
	}
}

<'a>t qsort(*(int ('a, 'a)) cmp, <'a>t l)
{
	'a m;
	
	switch l {
	case Nil[]:
		return l;
	case Cons[hd, r]:
		m = hd;
		
		<'a>t le = filter(fun bool ('a x) (cmp(x, m) <= 0), r);
		<'a>t gt = filter(fun bool ('a x) (cmp(x, m) > 0), r);
		return concat(qsort(cmp, le), Cons[hd, qsort(cmp, gt)]);
	}
}

<'a>t cons1('a d1)
{
	return List::Cons[d1, List::Nil];
}

<'a>t cons2('a d1, 'a d2)
{
	return List::Cons[d1, List::Cons[d2, List::Nil]];
}

<'a>t cons3('a d1, 'a d2, 'a d3)
{
	return List::Cons[d1, List::Cons[d2, List::Cons[d3, List::Nil]]];
}

<'a>t cons4('a d1, 'a d2, 'a d3, 'a d4)
{
	return List::Cons[d1, List::Cons[d2, List::Cons[d3, 
					List::Cons[d4, List::Nil]]]];
}

<'a>t rev(<'a>t l)
{
	<'a>t rev_tail(<'a>t acc, <'a>t lst)
	{
		switch lst {
		case Nil:
			return acc;
		case Cons[hd, tl]:
			return rev_tail(Cons[hd, acc], tl);
		}
	}

	return rev_tail(Nil, l);
}

<'a>t nil()
{
	return Nil;
}

<'a>t cons('a hd, <'a>t tl)
{
	return Cons[hd, tl];
}

bool exists(*(bool ('a)) f, <'a>t l)
{
	switch l {
	case Cons[hd, tl]:
		if (f(hd))
			return true;
		else
			return exists(f, tl);

	case Nil:
		return false;
	}
}

'a find(*(bool ('a)) f, <'a>t lst)
{
	switch lst {
	case Cons[x, xs]:
		if (f(x))
			return x;
		else
			return find(f, xs);
	case Nil:
		raise Not_found;
	}
}
